namespace SampleCandleStorageTesting
{
	using System;

	using StockSharp.Algo;
	using StockSharp.Algo.Candles;
	using StockSharp.Algo.Indicators;
	using StockSharp.Algo.Indicators.Trend;
	using StockSharp.Algo.Strategies;
	using StockSharp.BusinessEntities;

	class SmaStrategy : TimeFrameStrategy
	{
		private readonly CandleManager _candleManager;
		private bool _isShortLessThenLong;

		private DateTime _nextTime;

		public SmaStrategy(CandleManager candleManager, SimpleMovingAverage longSma, SimpleMovingAverage shortSma, TimeSpan timeFrame)
			: base(timeFrame)
		{
			_candleManager = candleManager;

			LongSma = longSma;
			ShortSma = shortSma;
		}

		public SimpleMovingAverage LongSma { get; private set; }
		public SimpleMovingAverage ShortSma { get; private set; }

		protected override void OnStarting()
		{
			//      
			_isShortLessThenLong = ShortSma.LastValue < LongSma.LastValue;

			//     
			_nextTime = TimeFrame.GetCandleBounds(Trader).Max;

			base.OnStarting();
		}

		protected override ProcessResults OnProcess()
		{
			//      
			if (ProcessState == ProcessStates.Stopping)
			{
				//   
				CancelActiveOrders();

				//        ,   ProcessResults.Stop
				return ProcessResults.Stop;
			}

			//       ,  ,    5-.
			if (Trader.MarketTime < _nextTime)
			{
				//  ProcessResults.Continue,         ,     .
				return ProcessResults.Continue;
			}

			//   
			var candle = _candleManager.GetTimeFrameCandle(Security, TimeFrame, _nextTime - TimeFrame);

			//     (      -),     .
			if (candle == null)
			{
				//    10     ,      ,
				//     5-  ,     
				if ((Trader.MarketTime - _nextTime) > TimeSpan.FromSeconds(10))
					_nextTime = TimeFrame.GetCandleBounds(Trader.MarketTime).Max;

				return ProcessResults.Continue;
			}

			_nextTime += TimeFrame;

			//   
			LongSma.Process((DecimalIndicatorValue)candle.ClosePrice);
			ShortSma.Process((DecimalIndicatorValue)candle.ClosePrice);

			//      
			var isShortLessThenLong = ShortSma.LastValue < LongSma.LastValue;

			//   
			if (_isShortLessThenLong != isShortLessThenLong)
			{
				//     ,  , , .
				var direction = isShortLessThenLong ? OrderDirections.Sell : OrderDirections.Buy;

				//  
				var order = this.CreateOrder(direction, Security.GetMarketPrice(direction), Volume);

				//   (  -  )
				// RegisterOrder(order);

				//   ( )
				var strategy = new MarketQuotingStrategy(order, new Unit(), new Unit());
				ChildStrategies.Add(strategy);

				//      
				_isShortLessThenLong = isShortLessThenLong;
			}

			return ProcessResults.Continue;
		}
	}
}